// Energy filter: sum consecutive time-samples and peak-detect
// In Rev2, Sum is changed to a wire (in Rev1 was a reg) 

`define TPG_Width   10
module EnFilterRev2
(  clk, DataIn, Peak , PeakDetected  );

input clk;
input  [`TPG_Width - 1 :0] DataIn;
output [`TPG_Width - 1 :0] Peak;
output PeakDetected;

reg [`TPG_Width - 1 :0] DataIn_d, Sum_d, Sum_2d;

always@(posedge clk)   DataIn_d <= DataIn;
wire [`TPG_Width - 1 :0] Sum = DataIn + DataIn_d;

// The following code detects a peak in the incoming data ("Sum"):
// it takes three consecutive data samples and if the middle one
// has the largest value, that sample is taken as the peak and sent out.
// If there is a "plateau", it selects only the first sample (it would be
// a little complicate to select them all, as it could be confused 
// with a plateau of relative minima).
// If a peak is not detected, a zero is sent out.
always@(posedge clk)begin
   Sum_d <= Sum;
   Sum_2d <= Sum_d;
 end

   // Note the "bigger than or equal to"
wire PeakDetected = (Sum_d >= Sum) && (Sum_d > Sum_2d);
wire [`TPG_Width - 1 :0] Peak = PeakDetected ? Sum_d : 0;

endmodule