// Description : Array of 8 synchronization FIFOs and releted controlers. 
`timescale 1ns / 10ps

module FifoStage (
  clk2x, AsyncFifoRst, FifoRestart, LatTrim, full, empty, LW, WrEn, RdEn,
  Fifo1Status, Fifo2Status, Fifo3Status, Fifo4Status, 
  Fifo5Status, Fifo6Status, Fifo7Status, Fifo8Status,
  Ch1RxD,   Ch1RxEr,    Ch1RxDV,    Ch1_RX_Clk,  
  Ch2RxD,   Ch2RxEr,    Ch2RxDV,    Ch2_RX_Clk,
  Ch3RxD,   Ch3RxEr,    Ch3RxDV,    Ch3_RX_Clk,
  Ch4RxD,   Ch4RxEr,    Ch4RxDV,    Ch4_RX_Clk,
  Ch5RxD,   Ch5RxEr,    Ch5RxDV,    Ch5_RX_Clk,
  Ch6RxD,   Ch6RxEr,    Ch6RxDV,    Ch6_RX_Clk,    
  Ch7RxD,   Ch7RxEr,    Ch7RxDV,    Ch7_RX_Clk,
  Ch8RxD,   Ch8RxEr,    Ch8RxDV,    Ch8_RX_Clk,
  Ch1RxD_o,   Ch1RxEr_o,    Ch1RxDV_o,      
  Ch2RxD_o,   Ch2RxEr_o,    Ch2RxDV_o,    
  Ch3RxD_o,   Ch3RxEr_o,    Ch3RxDV_o,   
  Ch4RxD_o,   Ch4RxEr_o,    Ch4RxDV_o,    
  Ch5RxD_o,   Ch5RxEr_o,    Ch5RxDV_o,    
  Ch6RxD_o,   Ch6RxEr_o,    Ch6RxDV_o,        
  Ch7RxD_o,   Ch7RxEr_o,    Ch7RxDV_o,    
  Ch8RxD_o,   Ch8RxEr_o,    Ch8RxDV_o     );
  
  input clk2x, AsyncFifoRst, FifoRestart, LatTrim;
  input [15:0]  Ch1RxD,  Ch2RxD,  Ch3RxD,  Ch4RxD,  
                Ch5RxD,  Ch6RxD,  Ch7RxD,  Ch8RxD;   
  input    Ch1RxEr, Ch1RxDV, Ch1_RX_Clk,
           Ch2RxEr, Ch2RxDV, Ch2_RX_Clk,
           Ch3RxEr, Ch3RxDV, Ch3_RX_Clk,
           Ch4RxEr, Ch4RxDV, Ch4_RX_Clk,
           Ch5RxEr, Ch5RxDV, Ch5_RX_Clk,
           Ch6RxEr, Ch6RxDV, Ch6_RX_Clk,
           Ch7RxEr, Ch7RxDV, Ch7_RX_Clk,
           Ch8RxEr, Ch8RxDV, Ch8_RX_Clk;            

  output[15:0]  Ch1RxD_o,  Ch2RxD_o,  Ch3RxD_o,  Ch4RxD_o,  
                Ch5RxD_o,  Ch6RxD_o,  Ch7RxD_o,  Ch8RxD_o;   
  output    Ch1RxEr_o, Ch1RxDV_o,   Ch2RxEr_o, Ch2RxDV_o,
            Ch3RxEr_o, Ch3RxDV_o,   Ch4RxEr_o, Ch4RxDV_o,
            Ch5RxEr_o, Ch5RxDV_o,   Ch6RxEr_o, Ch6RxDV_o,
            Ch7RxEr_o, Ch7RxDV_o,   Ch8RxEr_o, Ch8RxDV_o;  
  output [3:0] Fifo1Status, Fifo2Status, Fifo3Status, Fifo4Status, 
               Fifo5Status, Fifo6Status, Fifo7Status, Fifo8Status;   
  output[8:1]  empty, full, LW, WrEn, RdEn;
  wire [17:0]  pad1,pad2,pad3,pad4,pad5,pad6,pad7,pad8;

fifo_ic_v2    SynchronizFifo1 (
   .read_clock_in(clk2x),       .write_clock_in(Ch1_RX_Clk),
   .read_enable_in(RdEn[1]),    .write_enable_in(WrEn[1]),
   .write_data_in({18'b0, Ch1RxDV, Ch1RxEr, Ch1RxD}), 
   .read_data_out({pad1, Ch1RxDV_o, Ch1RxEr_o, Ch1RxD_o}),
   .full_out(full[1]), .empty_out(empty[1]),
   .fifostatus_out(Fifo1Status), .fifo_gsr_in(AsyncFifoRst)  );

FifoCtr       FifoCtr1  (
   .clk2x(clk2x), .RX_Clk(Ch1_RX_Clk), .RX_ER(Ch1RxEr), .LatTrim(LatTrim), 
   .full(full[1]), .empty(empty[1]), .FifoStatus(Fifo1Status), .LW(LW[1]),  
   .FifoRestart(FifoRestart), .WrEn(WrEn[1]), .RdEn(RdEn[1])  );
    
fifo_ic_v2    SynchronizFifo2 (
   .read_clock_in(clk2x),       .write_clock_in(Ch2_RX_Clk),
   .read_enable_in(RdEn[2]),    .write_enable_in(WrEn[2]),
   .write_data_in({18'b0, Ch2RxDV, Ch2RxEr, Ch2RxD}), 
   .read_data_out({pad2, Ch2RxDV_o, Ch2RxEr_o, Ch2RxD_o}), 
   .full_out(full[2]), .empty_out(empty[2]),
   .fifostatus_out(Fifo2Status), .fifo_gsr_in(AsyncFifoRst)  );
   
FifoCtr       FifoCtr2  (
   .clk2x(clk2x), .RX_Clk(Ch2_RX_Clk), .RX_ER(Ch2RxEr), .LatTrim(LatTrim), 
   .full(full[2]), .empty(empty[2]), .FifoStatus(Fifo2Status), .LW(LW[2]),  
   .FifoRestart(FifoRestart),   .WrEn(WrEn[2]),   .RdEn(RdEn[2])    );
    
fifo_ic_v2    SynchronizFifo3 (
   .read_clock_in(clk2x), .write_clock_in(Ch3_RX_Clk), 
   .read_enable_in(RdEn[3]),  .write_enable_in(WrEn[3]), 
   .write_data_in({18'b0, Ch3RxDV, Ch3RxEr, Ch3RxD}), 
   .read_data_out({pad3, Ch3RxDV_o, Ch3RxEr_o, Ch3RxD_o}), 
   .full_out(full[3]), .empty_out(empty[3]),
   .fifostatus_out(Fifo3Status),  .fifo_gsr_in(AsyncFifoRst)  );
    
FifoCtr       FifoCtr3  (
   .clk2x(clk2x), .RX_Clk(Ch3_RX_Clk), .RX_ER(Ch3RxEr), .LatTrim(LatTrim), 
   .full(full[3]), .empty(empty[3]), .FifoStatus(Fifo3Status), .LW(LW[3]),  
   .FifoRestart(FifoRestart),   .WrEn(WrEn[3]),  .RdEn(RdEn[3])    );
    
fifo_ic_v2    SynchronizFifo4 (
   .read_clock_in(clk2x),     .write_clock_in(Ch4_RX_Clk), 
   .read_enable_in(RdEn[4]),   .write_enable_in(WrEn[4]), 
   .write_data_in({18'b0, Ch4RxDV, Ch4RxEr, Ch4RxD}), 
   .read_data_out({pad4, Ch4RxDV_o, Ch4RxEr_o, Ch4RxD_o}), 
   .full_out(full[4]), .empty_out(empty[4]),
   .fifostatus_out(Fifo4Status), .fifo_gsr_in(AsyncFifoRst)  );
    
FifoCtr       FifoCtr4  (
   .clk2x(clk2x), .RX_Clk(Ch4_RX_Clk), .RX_ER(Ch4RxEr), .LatTrim(LatTrim), 
   .full(full[4]), .empty(empty[4]), .FifoStatus(Fifo4Status), .LW(LW[4]),  
   .FifoRestart(FifoRestart),   .WrEn(WrEn[4]),  .RdEn(RdEn[4])    );
    
fifo_ic_v2    SynchronizFifo5 (
   .read_clock_in(clk2x), .write_clock_in(Ch5_RX_Clk), 
   .read_enable_in(RdEn[5]),  .write_enable_in(WrEn[5]), 
   .write_data_in({18'b0, Ch5RxDV, Ch5RxEr, Ch5RxD}), 
   .read_data_out({pad5, Ch5RxDV_o, Ch5RxEr_o, Ch5RxD_o}), 
   .full_out(full[5]), .empty_out(empty[5]), 
   .fifostatus_out(Fifo5Status), .fifo_gsr_in(AsyncFifoRst)  );
    
FifoCtr       FifoCtr5  (
   .clk2x(clk2x), .RX_Clk(Ch5_RX_Clk), .RX_ER(Ch5RxEr), .LatTrim(LatTrim), 
   .full(full[5]), .empty(empty[5]), .FifoStatus(Fifo5Status), .LW(LW[5]),  
   .FifoRestart(FifoRestart),   .WrEn(WrEn[5]),  .RdEn(RdEn[5])    );
    
fifo_ic_v2    SynchronizFifo6 (
   .read_clock_in(clk2x),    .write_clock_in(Ch6_RX_Clk), 
   .read_enable_in(RdEn[6]),  .write_enable_in(WrEn[6]), 
   .write_data_in({18'b0, Ch6RxDV, Ch6RxEr, Ch6RxD}), 
   .read_data_out({pad6, Ch6RxDV_o, Ch6RxEr_o, Ch6RxD_o}), 
   .full_out(full[6]), .empty_out(empty[6]), 
   .fifostatus_out(Fifo6Status), .fifo_gsr_in(AsyncFifoRst)  );

FifoCtr       FifoCtr6  (
   .clk2x(clk2x), .RX_Clk(Ch6_RX_Clk), .RX_ER(Ch6RxEr), .LatTrim(LatTrim), 
   .full(full[6]), .empty(empty[6]), .FifoStatus(Fifo6Status), .LW(LW[6]),  
   .FifoRestart(FifoRestart),   .WrEn(WrEn[6]),  .RdEn(RdEn[6])     );
    
fifo_ic_v2    SynchronizFifo7 (
   .read_clock_in(clk2x),    .write_clock_in(Ch7_RX_Clk), 
   .read_enable_in(RdEn[7]),    .write_enable_in(WrEn[7]), 
   .write_data_in({18'b0, Ch7RxDV, Ch7RxEr, Ch7RxD}), 
   .read_data_out({pad7, Ch7RxDV_o, Ch7RxEr_o, Ch7RxD_o}), 
   .full_out(full[7]), .empty_out(empty[7]),
   .fifostatus_out(Fifo7Status),  .fifo_gsr_in(AsyncFifoRst)   );

FifoCtr       FifoCtr7  (
   .clk2x(clk2x), .RX_Clk(Ch7_RX_Clk), .RX_ER(Ch7RxEr), .LatTrim(LatTrim), 
   .full(full[7]), .empty(empty[7]), .FifoStatus(Fifo7Status), .LW(LW[7]),  
   .FifoRestart(FifoRestart),   .WrEn(WrEn[7]),  .RdEn(RdEn[7])    );
    
fifo_ic_v2    SynchronizFifo8 (
   .read_clock_in(clk2x),    .write_clock_in(Ch8_RX_Clk),
   .read_enable_in(RdEn[8]),   .write_enable_in(WrEn[8]), 
   .write_data_in({18'b0, Ch8RxDV, Ch8RxEr, Ch8RxD}), 
   .read_data_out({pad8, Ch8RxDV_o, Ch8RxEr_o, Ch8RxD_o}), 
   .full_out(full[8]), .empty_out(empty[8]),
   .fifostatus_out(Fifo8Status),  .fifo_gsr_in(AsyncFifoRst)   );
    
FifoCtr       FifoCtr8  (
   .clk2x(clk2x),   .RX_Clk(Ch8_RX_Clk),   .RX_ER(Ch8RxEr), .LatTrim(LatTrim), 
   .full(full[8]), .empty(empty[8]), .FifoStatus(Fifo8Status), .LW(LW[8]),  
   .FifoRestart(FifoRestart),   .WrEn(WrEn[8]),  .RdEn(RdEn[8])    );
    
endmodule